/** *************************************************************************
 *
 * @file	display.c
 * @brief	This is LCD display file for S1C17W22_W23.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 04/25/2014    1.00
 *				- First revision.
 *
 ************************************************************************* */


#include <string.h>
#include "lcd_font.h"
#include "display.h"

/*************************************************************
 * SelectLcdFont function.
 * This function select a Lcd Font.
 *
 * param: unsgined char  ch  Select key.
 * return: char *  Pointer of Lcd Font in lcdFontArray.
 * history: 2008/04/07	start
 ************************************************************/
static const unsigned long * selectLcdFont(unsigned char ch) {
	if((FIRST_NO_OF_ASCII_CODE <= ch) && (ch <= END_NO_OF_ASCII_CODE)) {
		return lcdFontArray[ch - FIRST_NO_OF_ASCII_CODE];
	} else {
		return lcdFontArray[0];
	}
}



/*************************************************************
 * displayChar function.
 * This function displays 1 Charactor
 * at specified Lcd line and specified charactor row.
 *
 * param: unsigned char  lineNo  Lcd line No. which you want to display. (0 - 7)
 * param: unsigned char  charNo  Lcd Charactor row No. which you want to display. (0 - 9)
 * param: unsgined char   ch      Display charactors.
 * return: void
 * history: 2008/11/10	start
 ************************************************************/
void displayChar(unsigned char lineNo, unsigned char charNo, unsigned char ch) {
	unsigned long  j, *ll;
	unsigned char *displayStartAdd, c;

	if ((0 < lineNo) || (9 < charNo)) {
		// it is over flow, so return.
		return;
	}
	displayStartAdd = (unsigned char*) displayMemory[lineNo] + (DISPLAY_CH_ROW_SIZE * charNo) * 4;
	ll = (unsigned long *) selectLcdFont(ch);
	for (j = 0; j < DISPLAY_CH_ROW_SIZE; j++) {
		c = (unsigned char) *ll;
		ll++;
		*displayStartAdd = c;
		displayStartAdd += 4;
	}
}

/*************************************************************
 * DisplayStr function.
 * This function displays String(0 - 9 charactors)
 * at specified Lcd line.
 *
 * param: unsigned char  lineNo  Lcd line No. which you want to display. (0 - 7)
 * param: unsinged char*  str     Display String(0-9 charactors).
 * return: void
 * history: 2008/11/10	start
 ************************************************************/
void displayStr(unsigned char lineNo, unsigned char charNo, unsigned char *str) {
	unsigned long  i, j;
	unsigned char *displayStartAdd, c;

	for(i = charNo; i < 11; i++) {

		unsigned long	*ll;

		if(*str == '\0') {
			break;	// end of String.
		}
		displayStartAdd = (unsigned char*) displayMemory[lineNo] + (DISPLAY_CH_ROW_SIZE * i) * 4;
		ll = (unsigned long *) selectLcdFont(*str++);
		for (j = 0; j < DISPLAY_CH_ROW_SIZE; j++) {
			c = (unsigned char) *ll;
			ll++;
			*displayStartAdd = c;
			displayStartAdd += 4;
		}

	}
}


/*************************************************************
 * ClearLcdLine function.
 * This function clear a specified Lcd line.
 *
 * param: char  lineNo  Lcd line No. which you want to clear. (0 - 7)
 * return: void
 * history: 2008/11/10	start
 ************************************************************/
void clearLcdLine(unsigned char lineNo) {
	if(0 < lineNo) {
		// it is over flow, so return.
		return;
	}
	memset((unsigned long *)displayMemory[lineNo], 0x00, MAX_SEG_SIZE*4+16);
}


/*************************************************************
 * DrawVertLcd function.
 * This function draw a vertical line on Lcd.
 *
 * param:	hPos:	horizontal position (24-63)
 * 			vData:	data for vertical line (16bit, LSB=top)
 * return: void
 * history: 2016/07/26	start
 ************************************************************/
void drawVertLcd(unsigned short hPos, unsigned short vData) {

	unsigned short	*p;

	p = (unsigned short *) ( displayMemory[0] + 4 * hPos );
	*p = vData;
}

