/** *************************************************************************
 *
 * @file	main.c
 * @brief	This is PPG demonstration program for S1C17W23 application note.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 07/20/2016    1.00
 *				- First revision.
 *			- 08/25/2016    1.10
 *				- 2nd revision.
 *
 ************************************************************************* */


#include <string.h>

#include "crt0.h"
#include "init.h"
#include "ad12.h"
#include "display.h"
#include "osc.h"
#include "t16_ch0.h"
#include "t16_ch1.h"
#include "t16_ch3.h"
#include "ppg_data.h"


/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector05_handler, intPport);		// PPORT interrupt
C17_INTERRUPT_HANDLER(_vector09_handler, intT16Ch0);	// T16 Ch.0 interrupt
C17_INTERRUPT_HANDLER(_vector26_handler, intAdc12Ch0);	// ADC12A interrupt
#endif /* CRT0_LINKED */


/*
 * 														LCD
 * P04	J1-36	I	SW4		Start/Stop					R(running)/S(stop)
 * P05	J1-35	I	SW3		Gain Low/High				L/H
 * P06	J1-34	I	SW2		LPF ON/OFF					F/-
 * P07	J1-33	I	SW1		Continuous/Intermittent		C/I
 *
 * P10	J1-31	O	0: LED OFF  1: LED ON
 * P15	J1-26	O	0: Hold  1: Sampling
 * P16	J1-25	O	0: High Gain  1: Low Gain
 * P17	J1-24	O	0: Vref OFF  1: Vref ON
 * P20	J1-22	O	0: LPF OFF  1: LPF ON
 *
 * P12	J1-29	ADIN00
 * P13	J1-28	VREFA0
 *
 * P13	J1-28	#BZOUT	... There must not be influence.
 * P14	J1-27	BZOUT
 *
 * Peripherals	(Interrupt level)
 *
 *   T16Ch0		(1)		Interval timer for sampling.
 *   T16Ch1		(-)		Timer for sampling delay and pulse width.
 *   T16Ch3		(-)		Clock for ADC12A.
 *   Adc12aCh0	(3)		Convert photo diode signal level.
 *
 */

#define OSC3_FREQ						OSC3_4MHZ

#define SAMPLING_DIV					0				// Clock division ratio selection (setting 1/1)
#define SAMPLING_TIMER_COEF			654				// Reload data  ( 654 + 1 ) / 32.768 = 19.99ms
#define SAMPLING_DELEY_DIV				0				// Clock division ratio selection (setting 1/1)
#define ADC12_CLOCK_COEF				1				// ( 4MHz / 1 ) / ( 1 + 1 ) = 2MHz ... ADC12A: fADCLK(max) = 2,000kHz

#define SAMPLING_DELAY					0
#define CALC_HR_CYCLE					51				// 20ms * 51 = 1.02s  (CALC_HR_CYCLE should be integral multiples of REDRAW_GRAPH_CYCLE)
#define REDRAW_GRAPH_CYCLE				3				// 20ms * 3

#define NUM_FRAME_INIT_VAL				12				// 60BPM: (1000ms/20ms)/4 = 12.5

// It is necessary to adjust the following parameters.
#define NUM_FRAME_MIN					5				// 150BPM: (400ms/20ms)/4 = 5
#define NUM_FRAME_MAX					18				// 40BPM: (1,500ms/20ms)/4 = 18.75
#define FIND_PEAK_CONT_NUM				6


int main(void);
unsigned short calcHeartRate (void);
void getMinMax(void);
unsigned short checkHeartBeatTiming(void);
void execFir(void);
void calcCC(void);

void initLcd(void);
void initPport(void);
void initT16Ch0(void);
void initT16Ch1(void);
void initT16Ch3(void);

void val2Str4(unsigned char *dst, unsigned short digit, unsigned short zeroSup, unsigned short val);
void dispHeartMark(unsigned short mark);
void dispHeartRate(unsigned short bpm);
void dispStat(void);
void setP1Base(void);
void clearGraph(void);

void intPport(void)__attribute__ ((interrupt_handler));
void intT16Ch0(void)__attribute__ ((interrupt_handler));
void intAdc12Ch0(void)__attribute__ ((interrupt_handler));

#define FIR_COEF_NUM		15

const signed short firCoef[FIR_COEF_NUM] = {
	10486,		//  0:  0.16		* 65536
	 9940,		//  1:  0.151671458
	 8426,		//  2:  0.128570127
	 6277,		//  3:  0.095781955
	 3938,		//  4:  0.060091914
	 1839,		//  5:  0.028064679
	  285,		//  6:  0.004351915
	 -606,		//  7: -0.009244721
	 -900,		//  8: -0.01372657
	 -787,		//  9: -0.012002826
	 -496,		// 10: -0.007568267
	 -215,		// 11: -0.003277083
	  -41,		// 12: -0.000629929
	   17,		// 13:  0.000263222
	   11,		// 14:  0.000170057
				// 15:  0
};

unsigned char strDisp[11];

PPG_DATA	pd;


int main (void)
{

	/// Set interrupt level.
	ILVPPORT		= 1;		// Set GPIO interrupt level
	ILVT16_0		= 3;		// Set T16Ch0 interrupt level
	ILVADC12_0		= 5;		// Set Adc12aCh0 interrupt level;  must be > ILVT16_0

	pd.running = 0;				// 0: stop
	pd.interm = 0;				// 0: continuous
	pd.highGain = 0;			// 0: low gain
	pd.lpf = 1;					// 1: noise filter ON

	pd.samplingDelay = SAMPLING_DELAY;
	pd.numFrame = NUM_FRAME_INIT_VAL;

	startOsc1();				// start OSC1 for LCD and T16Ch0

	initLcd();
	clearLcdLine(0);
	clearLcdLine(1);

	initPport();				// initialize PPORT for SW1-4 interrupt

	initT16Ch0();				// Initialize T16Ch0 for interval timer
	initT16Ch1();				// Initialize T16Ch1 for delay timer
	initT16Ch3();				// Initialize T16Ch3 for ADC12A's clock

	// Assign port to ADC12.
	P1MODSEL		= 0x000C;	// ----.----.----.11--	P13=VREFA0 P12=ADIN00
	P1FNCSEL		= 0x00A0;	// ----.----.1010.----

	initAdc();					// Initialize Adc12aCh0 for photo diode signal level converter

	ADC12_0TRG	= 0x0014;		// --00.0000.0001.-100	SMPCLK=4(8cycle) CNVTRG=1(T16) CNVMD=0(once) STMD=0(right) STAAIN=0 ENDAIN=0

	// Set P10, P15, P16, P17, P20 for output.
	P1IOEN			= 0x00E1;	// ----.----.111-.---1
	P2IOEN_P20OEN	= 1;

	dispHeartMark( 1 );
	dispHeartRate( 0 );
	dispStat();

	changeSysClkOscto1();		// change clock to OSC1
	stopIOsc();					// stop IOSC

	while (1) {
		asm("ei");
		asm("halt");
		asm("nop");
	}

	return (0);
}

unsigned short calcHeartRate (void)
{
	unsigned short	i, dataPtr, lastRes = 0, contP = 0, contM = 0, distNum = 0, sum = 0;
	signed short	dist;

	dataPtr = pd.dataPtr;
	for ( i = 0; i < ( DATA_NUM - 1 ); i++ ) {
		if ( ++dataPtr >= DATA_NUM ) {
			dataPtr = 0;
		}
		if ( pd.ccRes[dataPtr] >= pd.ccRes[dataPtr + 1] ) {
			contP++;
		} else {
			contM++;
			contP = 0;
		}
		if ( contM >= FIND_PEAK_CONT_NUM && contP >= FIND_PEAK_CONT_NUM ) {		// find minimum point
			contM = 0;
			contP = 0;
			if ( distNum >= 1 ) {
				dist = dataPtr + 1 - lastRes;
				if ( dist < 0 ) {
					dist += DATA_NUM;
				}
				sum += dist;
			}
			lastRes = dataPtr + 1;
			distNum++;
		}
	}

	if ( distNum >= 2 ) {

		float	fBpm;
		unsigned short	newNumFrame;

		// ts = 20ms --> bpm = (60,000ms/20ms) / distAv
		fBpm = 3000.0 / ( (float) sum / (float) ( distNum - 1 ) );
		pd.heartRate = fBpm + 0.5;

		// newNumFrame = hRFromSystole / 4
		newNumFrame = ( pd.hRFromSystole + 2 ) / 4;

		// 60BPM: (1000ms/20ms)/4=12.5  75BPM: (800ms/20ms)/4=10
		if ( NUM_FRAME_MIN <= newNumFrame && newNumFrame <= NUM_FRAME_MAX ) {
			if ( newNumFrame > pd.numFrame ) {
				pd.numFrame++;
			} else if ( newNumFrame < pd.numFrame ) {
				pd.numFrame--;
			}
		}

		return 1;
	}

	return 0;

}

void getMinMax (void)
{
	unsigned short	i, dataPtr;

	pd.rawMinVal = 4096;
	pd.rawMaxVal = 0;
	dataPtr = pd.dataPtr;
	for ( i = 0; i < DATA_NUM; i++ ) {
		if ( ++dataPtr >= DATA_NUM ) {
			dataPtr = 0;
		}

		if ( pd.rawData[dataPtr] < pd.rawMinVal ) {
			pd.rawMinVal = pd.rawData[dataPtr];
		}
		if ( pd.rawData[dataPtr] > pd.rawMaxVal ) {
			pd.rawMaxVal = pd.rawData[dataPtr];
		}
	}

	pd.rawRange = pd.rawMaxVal - pd.rawMinVal;
	pd.rawSystoleLevel = ( pd.rawMaxVal + pd.rawMinVal ) >> 1 ;		// divide by 2
}

unsigned short checkHeartBeatTiming(void)
{
	if ( pd.rawData[pd.dataPtr] <= pd.rawSystoleLevel ) {
		return 1;
	}

	return 0;

}

void execFir(void)
{
	signed short	ptr;
	unsigned short	i;
	signed long	res, sum = 0;

	ptr = pd.dataPtr;
	for ( i = 0; i < FIR_COEF_NUM; i++ ) {
		if ( firCoef[i] >= 0 ) {
			res = pd.rawData[ptr];
			res *= firCoef[i];
//			res = pd.rawData[ptr] * firCoef[i];	// NG
			sum += res >> 16;
		} else {
			res = pd.rawData[ptr];
			res *= ( -firCoef[i] );
//			res = pd.rawData[ptr] * ( -firCoef[i] );	// NG
			sum -= res >> 16;
		}
		if ( --ptr < 0 ) {
			ptr = DATA_NUM - 1;
		}
	}
	pd.firRes[pd.dataPtr] = sum;
}

void calcCC(void)
{
	unsigned short	i;
	signed short	t, y;

	y = 0;
	t = pd.dataPtr + 1;
	if ( t < 0 ) {
		t += DATA_NUM;
	}

	for ( i = 0; i < pd.numFrame; i++ ) {
		y -= pd.firRes[t];
		if ( ++t >= DATA_NUM ) {
			t = 0;
		}
	}
	for ( i = 0; i < ( 2 * pd.numFrame ); i++ ) {
		y += pd.firRes[t];
		if ( ++t >= DATA_NUM ) {
			t = 0;
		}
	}
	for ( i = 0; i < pd.numFrame; i++ ) {
		y -= pd.firRes[t];
		if ( ++t >= DATA_NUM ) {
			t = 0;
		}
	}
	pd.ccRes[pd.dataPtr] = y;
}

void initLcd (void)
{
	// P23..P27 use as COM8..COM12
	// P2SELy = 1
	// P2yMUX = 0x3
	*(volatile unsigned int *)(0x422c) = 0x00f8;
	*(volatile unsigned int *)(0x422e) = 0xffc0;

	// P30..P37 use as COM8..COM15, SEG63..SEG59
	//	P3SELy = 1
	//	P3yMUX = 0x3
	*(volatile unsigned int *)(0x423c) = 0x00ff;
	*(volatile unsigned int *)(0x423e) = 0xffff;

	// P40..P43 use as SEG58..SEG54
	// P4SELy = 1
	// P4yMUX = 0x3
	*(volatile unsigned int *)(0x424c) = 0x001f;
	*(volatile unsigned int *)(0x424e) = 0x03ff;

	// LCD24CLK.CLKSRC[1:0] = OSC1, LCD24CLK.CLKDIV[2:0] = any
	// LCD24CLK.DBRUN = 1
	*(volatile unsigned int *)(0x5400) = 0x0101;

	// LCD24CTL = 1
	*(volatile unsigned int *)(0x5402) = 0x0001;


	// LCD24TIM1.LDUTY[4:0] = 0x0f (=1/16)
	// LCD24TIM1.COMPOS = 0x0 (normal LCD pin assignment)
	// LCD24TIM1.FRMCNT[4:0] = 0x03 (=64Hz)
	*(volatile unsigned int *)(0x5404) = 0x030f;


	// LCD24TIM2.NLINE[4:0] = no use (default) ... no use negative drive default)
	// LCD24TIM2.BSTC[1:0] = 1/16 (default) ... boost clock 1/16
	// *(volatile unsigned int *)(0x5406) = 0x0100; ... default

	// LCD24PWR.VCEN=1 (=on)
	// LCD24PWR.VCSEL=0 (=VC1)
	// LCD24PWR.BISEL=1 (=1/4bias)
	// LCD24PWR.BSTEN=1 (=on)
	// LCD24PWR.LC[4:0]=13 (=1.165V)
	*(volatile unsigned int *)(0x5408) = 0x1519;

	// LCD24DSP.DSPC = 1 (normal)
	// LCD24DSP.DSPAR = 0
	// LCD24DSP.DSPREV = 1 ... reverse display off
	// LCD24DSP.COMREV = 1
	// LCD24DSP.SEGREV = 1
	*(volatile unsigned int *)(0x540a) = 0x0071;
}

void initPport (void)
{
	/*
	 * P04	J1-36	I	SW4		Start/Stop					R(running)/S(stop)
	 * P05	J1-35	I	SW3		Gain Low/High				L/H
	 * P06	J1-34	I	SW2		LPF ON/OFF					F/-
	 * P07	J1-33	I	SW1		Continuous/Intermittent		C/I
	 */

	unsigned short		pportInactCount = 0;

	P0INTCTL	= 0x0000;	// ----.----.0000.----  Disable interrupt

	// Set Chattering filter.
	if ( CLGOSC_OSC1EN == 1 ) {
		protectMisc( MSCPROT_PROTECT_OFF );		// Disable write-protect.

		PCLK_CLKSRC			= 1;	/// - Set clock source for chattering filter(OSC1).
		PCLK_CLKDIV			= 0x9;	/// - Set division ratio for clock source(1/(2*512) = 31.25msec).
		PCLK_DBRUN			= 1;	/// - Clock supplied in DEBUG mode.

		protectMisc( MSCPROT_PROTECT_ON );		// Enable write-protect.
	}

	P0CHATEN	= 0x00F0;	// ----.----.1111.----  Enable Chattering filter.
	P0RCTL		= 0xF0F0;	// 1111.----.1111.----  Enable pull-up resistor.

	while ( ++pportInactCount < 4096 ) {	// wait for PPORT chattering filter
		asm("nop");
	}

	P0INTCTL	= 0xF0F0;	// 1111.----.1111.----  Select edge(falling) and enable interrupt.
	P0IOEN		= 0xF000;	// 1111.----.0000.----  Set input port(Disable output port).
	P0INTF		= 0x00FF;	// ----.----.1111.****  Clear interrupt flag.
}

void initT16Ch0 (void)
{
	/// Stop T16 Ch0 Timer.
 	stopT16Ch0();

	T16_0CTL_MODEN		= 0;					/// - Stop supplying operating clock

	/// Configure the T16 Ch0 operating clock.
	T16_0CLK_CLKSRC 	= 1;					/// - Clock source selection(setting OSC1)
	T16_0CLK_CLKDIV 	= SAMPLING_DIV;			/// - Clock division ratio selection
	T16_0CLK_DBRUN		= 0;					/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_0CTL_MODEN		= 1;					/// - Start supplying operating clock.

	/// Select operation mode.
	T16_0MOD_TRMD		= 0;					/// - Set repeat mode.

	/// initialize interrupt.
	T16_0INTF			= 0x0001;				/// - Clear interrupt flag(T16_0INTF_UFIF = 1).
	T16_0INTE_UFIE		= 0;					/// - Disable interrupt.

	/// Preset reload data to counter.
	presetT16Ch0Count( SAMPLING_TIMER_COEF );

	return;
}

void initT16Ch1 (void)
{
	/// Stop T16 Ch1 Timer.
 	stopT16Ch1();

	T16_1CTL_MODEN		= 0;					/// - Stop supplying operating clock

	/// Configure the T16 Ch1 operating clock.
	T16_1CLK_CLKSRC 	= 2;					/// - Clock source selection(setting OSC3)
	T16_1CLK_CLKDIV 	= SAMPLING_DELEY_DIV;	/// - Clock division ratio selection
	T16_1CLK_DBRUN		= 0;					/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_1CTL_MODEN		= 1;					/// - Start supplying operating clock.

	/// Select operation mode.
	T16_1MOD_TRMD		= 1;					/// - Set one shot mode.

	/// initialize interrupt.
	T16_1INTF			= 0x0001;				/// - Clear interrupt flag(T16_1INTF_UFIF = 1).
	T16_1INTE_UFIE		= 0;					/// - Disable interrupt.

	return;
}

void initT16Ch3 (void)
{
	/// Stop T16 Ch3 Timer.
 	stopT16Ch3();

	T16_3CTL_MODEN		= 0;					/// - Stop supplying operating clock

	/// Configure the T16 Ch3 operating clock.
	T16_3CLK_CLKSRC 	= 2;					/// - Clock source selection(setting OSC3)
	T16_3CLK_CLKDIV 	= 0;					/// - Clock division ratio selection
	T16_3TR				= ADC12_CLOCK_COEF;		/// - Reload data
	T16_3CLK_DBRUN		= 0;					/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_3CTL_MODEN		= 1;					/// - Start supplying operating clock.

	/// Select operation mode.
	T16_3MOD_TRMD		= 0;					/// - Set repeat mode.

	/// initialize interrupt.
	T16_3INTF			= 0x0001;				/// - Clear interrupt flag(T16_3INTF_UFIF = 1).
	T16_3INTE_UFIE		= 0;					/// - Disable interrupt.

	return;
}

void val2Str4 (unsigned char *dst, unsigned short digit, unsigned short zeroSup, unsigned short val)
{
	unsigned char		number;
	unsigned short		findNon0 = 0;

	if (digit >= 4) {
		if (val < 5000) {
			for(number = '0' ; number <= '4' ; number++){
				if(val < 1000){
					break;
				} else {
					val	-= 1000;
				}
			}
		} else {
			val	-= 5000;
			for(number = '5' ; number <= '9' ; number++){
				if(val < 1000){
					break;
				} else {
					val	-= 1000;
				}
			}
		}
		if ( zeroSup ) {
			if ( number != '0' ) {
				findNon0 = 1;
			}
			if ( number == '0' && findNon0 == 0 ) {
				*dst++ = ' ';
			} else {
				*dst++ = number;
			}
		} else {
			*dst++ = number;
		}
	}

	if (digit >= 3) {
		if (val < 500) {
			for(number = '0' ; number <= '4' ; number++){
				if(val < 100){
					break;
				} else {
					val	-= 100;
				}
			}
		} else {
			val	-= 500;
			for(number = '5' ; number <= '9' ; number++){
				if(val < 100){
					break;
				} else {
					val	-= 100;
				}
			}
		}
		if ( zeroSup ) {
			if ( number != '0' ) {
				findNon0 = 1;
			}
			if ( number == '0' && findNon0 == 0 ) {
				*dst++ = ' ';
			} else {
				*dst++ = number;
			}
		} else {
			*dst++ = number;
		}
	}

	if (digit >= 2) {
		if(val < 50){
			for(number = '0' ; number <= '4' ; number++){
				if(val < 10){
					break;
				} else {
					val	-= 10;
				}
			}
		} else {
			val	-= 50;
			for(number = '5' ; number <= '9' ; number++){
				if(val < 10){
					break;
				} else {
					val	-= 10;
				}
			}
		}
		if ( zeroSup ) {
			if ( number != '0' ) {
				findNon0 = 1;
			}
			if ( number == '0' && findNon0 == 0 ) {
				*dst++ = ' ';
			} else {
				*dst++ = number;
			}
		} else {
			*dst++ = number;
		}
	}

	*dst++ = val + '0';
	*dst = '\0';

	return;
}

void dispHeartMark(unsigned short mark)
{
	if ( mark == 0 ) {
		displayChar( 0, 0, LCD_CHAR_SMALL_HEART );		// The heart shrank, and the beat was detected.
	} else {
		displayChar( 0, 0, LCD_CHAR_LARGE_HEART );
	}
}

void dispHeartRate(unsigned short bpm)
{
	if ( bpm == 0 ) {
		strcpy( strDisp, "  ?" );
	} else {
		val2Str4( strDisp, 3, 1, bpm );
	}
	displayStr(0, 1, strDisp);
}

void dispStat(void)
{
	/*
	 * 														LCD
	 * P04	J1-36	I	SW4		Start/Stop					R(running)/S(stop)
	 * P05	J1-35	I	SW3		Gain Low/High				L/H
	 * P06	J1-34	I	SW2		LPF ON/OFF					F/-
	 * P07	J1-33	I	SW1		Continuous/Intermittent		C/I
	 */

	if ( pd.running == 0 ) {	// 0: stop
		strDisp[0] = 'S';
	} else {					// 1: running
		strDisp[0] = 'R';
	}

	if ( pd.highGain == 0 ) {	// 0: low gain
		strDisp[1] = 'L';
	} else {					// 1: high gain
		strDisp[1] = 'H';
	}

	if ( pd.lpf == 0 ) {		// 0: noise filter OFF
		strDisp[2] = '-';
	} else {					// 1: noise filter ON
		strDisp[2] = 'F';
	}

	if ( pd.interm == 0 ) {		// 0: continuous
		strDisp[3] = 'C';
	} else {					// 1: intermittent
		strDisp[3] = 'I';
	}
	strDisp[4] = '\0';

	displayStr (1, 0, strDisp);
}

void setP1Base(void)
{
	/*
	 * P10	J1-31	O	0: LED OFF  1: LED ON
	 * P15	J1-26	O	0: Hold  1: Sampling
	 * P16	J1-25	O	0: High Gain  1: Low Gain
	 * P17	J1-24	O	0: Vref OFF  1: Vref ON
	 */

	pd.p1xBase = 0x8000;		// 1---.----.----.----  P17=1(Vref ON)

	if ( pd.highGain == 0 ) {
		pd.p1xBase |= 0x4000;	// -1--.----.----.----  P16=1(Low Gain)
	}
	if ( pd.interm ) {			// 1: intermittent
		pd.p1xBase |= 0x2000;	// --1-.----.----.----  P15=1(Sampling)
	}
}

void clearGraph(void)
{
	unsigned short	i;

	for ( i = 24; i < 64; i++ ) {
		drawVertLcd( i, 0x00 );
	}
}

void intPport(void)
{
	unsigned short	pXintf;

	if ( ( PINTFGRP & 0x0001 ) ) {		// P0INT

		pXintf = P0INTF;
		P0INTF = pXintf;		// clear interrupt flag

		/*
		 * 														LCD
		 * P04	J1-36	I	SW4		Start/Stop					R(running)/S(stop)
		 * P05	J1-35	I	SW3		Gain Low/High				L/H
		 * P06	J1-34	I	SW2		LPF ON/OFF					F/-
		 * P07	J1-33	I	SW1		Continuous/Intermittent		C/I
		 */

		setP1Base();

		// P20	J1-22	O	0: LPF OFF  1: LPF ON
		if ( pd.lpf ) {	// 1: noise filter ON
			P2DAT_P20OUT	= 0x01;				// P20	1: LPF ON
		} else {				// 0: noise filter OFF
			P2DAT_P20OUT	= 0x00;				// P20	0: LPF OFF
		}

		if ( ( pXintf & 0x10 ) ) {	// P04	J1-36	I	SW4		Start/Stop					R(running)/S(stop)
			if ( pd.running == 0 ) {	// 0: stop
				pd.running = 1;
				pd.availDataNum = 0;
				pd.calcHRIter = CALC_HR_CYCLE - 1;
				pd.heartRate = 0;
				if ( pd.interm == 0 ) {			// 0: continuous
					P1DAT		= ( pd.p1xBase | 0x2100 );	// --1-.---1.----.----  P10=1(LED ON)  P15=1(Sampling)
				}
				clearGraph();
				startT16Ch0();			// start T16Ch0
			} else {					// 1: running
				pd.running = 0;
				stopT16Ch0();
				P1DAT		= 0x4000;	// 010-.---0.----.----  P10=0(LED OFF)  P15=0(Hold)  P16=1(Low Gain)  P17=0(Vref OFF)
			}
		}

		if ( pd.running == 0 ) {	// 0: stop
			if ( ( pXintf & 0x20 ) ) {	// P05	J1-35	I	SW3		Gain Low/High				L/H
				if ( pd.highGain == 0 ) {	// 0: low gain
					pd.highGain = 1;
				} else {					// 1: high gain
					pd.highGain = 0;
				}
			}

			if ( ( pXintf & 0x40 ) ) {	// P06	J1-34	I	SW2		LPF ON/OFF					F/-
				if ( pd.lpf == 0 ) {	// 0: noise filter OFF
					pd.lpf = 1;
				} else {				// 1: noise filter ON
					pd.lpf = 0;
				}
			}

			if ( ( pXintf & 0x80 ) ) {	// P07	J1-33	I	SW1		Continuous/Intermittent		C/I
				if ( pd.interm == 0 ) {		// 0: continuous
					pd.interm = 1;
				} else {					// 1: intermittent
					pd.interm = 0;
				}
			}
		}
		dispStat();
	}
}

void intT16Ch0 (void)
{
	changeSysClkOscto3( OSC3_FREQ );	// change clock to OSC3

	T16_0INTF = 0x0001;					// clear interrupt flag

	startT16Ch3();

	/*
	 * P10	J1-31	0: LED OFF  1: LED ON
	 * P15	J1-26	0: Hold  1: Sampling
	 * P16	J1-25	0: High Gain  1: Low Gain
	 * P17	J1-24	0: Vref OFF  1: Vref ON
	 * P20	J1-22	0: LPF OFF  1: LPF ON
	 */

	if ( pd.interm ) {		// 1: intermittent

		P1DAT		= ( pd.p1xBase | 0x2100 );	// --1-.---1.----.----  P10=1(LED ON)  P15=1(Sampling)

		// delay for sample dark level
		if ( pd.samplingDelay != 0 ) {
			presetT16Ch1Count(pd.samplingDelay);
			startT16Ch1();
			while ( T16_1CTL_PRUN != 0 ) {
			}
		}

		// measure
		ADC12_0INTF = 0x0001;				// ----.----.----.---1	AD0CIF=1(clear)
		startAdc();

		do {
			asm("ei");
			asm("halt");
			asm("nop");
		} while ( pd.stage != 1 );

		stopAdc();
		P1DAT		= ( pd.p1xBase & ~0x2100 );	// --0-.---0.----.----  P10=0(LED OFF)  P15=0(Hold)

	} else {		// 0: continuous

		// measure
		ADC12_0INTF = 0x0002;				// ----.----.----.--10	AD0CIF=2(clear)
		startAdc();

		do {
			asm("ei");
			asm("halt");
			asm("nop");
		} while ( pd.stage != 1 );

		stopAdc();

	}

	pd.stage = 0;
	stopT16Ch3();

	pd.rawData[pd.dataPtr] = pd.measuredVal;
	if ( ++pd.availDataNum >= DATA_NUM ) {

		pd.availDataNum = DATA_NUM;

		execFir();
		calcCC();

		// refresh heart rate value
		if ( ++pd.calcHRIter == CALC_HR_CYCLE ) {

			pd.calcHRIter = 0;
			getMinMax();

			if ( calcHeartRate() ) {
				dispHeartRate( pd.heartRate );
			}
		}

		pd.systoleCycleCounter++;

		// draw heart mark
		if ( checkHeartBeatTiming() != 0 ) {	// The heart shrank, and the beat was detected.
			if ( pd.currHeartMark == 1 ) {
				dispHeartMark( 0 );
				pd.currHeartMark = 0;
				pd.hRFromSystole = pd.systoleCycleCounter;
				pd.systoleCycleCounter = 0;
			}
		} else {
			if ( pd.currHeartMark == 0 ) {
				dispHeartMark( 1 );
				pd.currHeartMark = 1;
			}
		}

		// draw graphics
		if ( pd.calcHRIter % REDRAW_GRAPH_CYCLE == 0 ) {

			unsigned short	i, j, vData, vLine;
			signed short	val, dataPtr, lastVal = 0;		// avoid warning

			dataPtr = pd.dataPtr - REDRAW_GRAPH_CYCLE * ( 64 - 24 );
			while ( dataPtr < 0 ) {
				dataPtr += DATA_NUM;
			}
			for (i = 24; i < 64; i++) {
				dataPtr += REDRAW_GRAPH_CYCLE;
				if ( dataPtr >= DATA_NUM ) {
					dataPtr = 0;
				}

				val = ( pd.rawData[dataPtr] - pd.rawMinVal );
				if ( val > 0 ) {
					// normalize from 0 to 15
					if ( pd.rawRange > 2048 ) {				// >= 2^11
						val >>= 8;							// divide by 256
					} else if ( pd.rawRange > 1024 ) {		// >= 2^10
						val >>= 7;							// divide by 128
					} else if ( pd.rawRange > 512 ) {		// >= 2^9
						val >>= 6;							// divide by 64
					} else if ( pd.rawRange > 256 ) {		// >= 2^8
						val >>= 5;							// divide by 32
					} else if ( pd.rawRange > 128 ) {		// >= 2^7
						val >>= 4;							// divide by 16
					} else if ( pd.rawRange > 64 ) {		// >= 2^6
						val >>= 3;							// divide by 8
					} else if ( pd.rawRange > 32 ) {		// >= 2^5
						val >>= 2;							// divide by 4
					} else if ( pd.rawRange > 16 ) {		// >= 2^4
						val >>= 1;							// divide by 2
					} else {								// < 2^4
						;									// divide by 1
					}
				} else {
					val = 0;
				}

				// The signal is reversed because it darkens most when the heart shrinks.
				if ( i == 24 ) {
					vData = 0x0001 << val;
					drawVertLcd( i, vData );
				} else {
					vLine = 0;
					if ( val == lastVal ) {
						vData = 0x0001 << val;
						drawVertLcd( i, vData );
					} else if ( val >= lastVal ) {
						for ( j = lastVal; j <= val; j++ ) {
							vLine |= 0x0001 << j;
						}
						drawVertLcd( i, vLine );
					} else {
						for ( j = val; j <= lastVal; j++ ) {
							vLine |= 0x0001 << j;
						}
						drawVertLcd( i, vLine );
					}
				}
				lastVal = val;
			}
		}
	}

	if ( T16_0INTF ) {
		pd.overRunTimes++;
	}

	changeSysClkOscto1();		// change clock to OSC1
	stopOsc3();

	if ( ++pd.dataPtr >= DATA_NUM ) {
		pd.dataPtr = 0;
	}

}

void intAdc12Ch0 (void)
{
	if ( ( ADC12_0INTF & 0x0100 ) != 0 ) {	// ----.---1.----.----	AD0OVIE==1

		// Trap overwrite error, then clear interrupt flags and restart ADC.
		ADC12_0INTF = 0x0101;				// ----.---1.----.---1	AD0OVIE=1(clear), AD0CIF=1(clear)
		startAdc();							///Start ADC12

	} else if ( ( ADC12_0INTF & 0x0001 ) != 0 ) {	// ----.----.----.---1	AD0CIF==1

		// Complete ADC.
		ADC12_0INTF = 0x0001;				// ----.----.----.---1	AD0CIF=1(clear)

		// read ADC.
		pd.measuredVal = ADC12_0AD0D;

		pd.stage = 1;

	}

	return;

}

