/****************************************************************************
 *
 * @file	lcd24a_reg.h
 * @brief	Definition of the LCD Driver(LCD24A) Register.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2013-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 10/04/2013    1.00
 *				- First revision.
 *			- 03/27/2014    1.10
 *				- Modify register definition.
 *
 ***************************************************************************/
#ifndef	LCD24A_REG_H__
#define	LCD24A_REG_H__

#ifdef	__cplusplus
extern "C" {
#endif


/// LCD24A Clock Control Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		CLKSRC		: 2;	///< These bits select the clock source of the LCD24A.
		volatile unsigned short		reserved_1	: 2;	///< Reserved.
		volatile unsigned short		CLKDIV		: 3;	///< These bits select the division ratio of the LCD24A operating clock.
		volatile unsigned short		reserved_2	: 1;	///< Reserved.
		volatile unsigned short		DBRUN		: 1;	///< This bit sets whether the LCD24A operating clock is supplied in DEBUG mode or not.
		volatile unsigned short		reserved_3	: 7;	///< Reserved.
	} bit;
} LCD24CLK_REG;

/// LCD24A Control Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		MODEN		: 1;	///< This bit enables the LCD24A operations.
		volatile unsigned short		reserved_1	:15;	///< Reserved.
	} bit;
} LCD24CTL_REG;

/// LCD24A Timing Control Register 1.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		LDUTY		: 5;	///< These bits set the drive duty.
		volatile unsigned short		COMPOS		: 1;	///< This bit configures the COM pin layout when 1/1 to 1/16 duty is selected.
		volatile unsigned short		reserved_1	: 2;	///< Reserved.
		volatile unsigned short		FRMCNT		: 5;	///< These bits set the frame frequency.
		volatile unsigned short		reserved_2	: 3;	///< Reserved.
	} bit;
} LCD24TIM1_REG;

/// LCD24A Timing Control Register 2.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		NLINE		: 5;	///< These bits enable the n-line inverse AC drive function and set the number of inverse lines.
		volatile unsigned short		reserved_1	: 3;	///< Reserved.
		volatile unsigned short		BSTC		: 2;	///< These bits select the booster clock frequency for the LCD voltage booster.
		volatile unsigned short		reserved_2	: 6;	///< Reserved.
	} bit;
} LCD24TIM2_REG;

/// LCD24A Power Control Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		VCEN		: 1;	///< This bit turns the LCD voltage regulator on and off.
		volatile unsigned short		VCSEL		: 1;	///< This bit sets the LCD voltage regulator output (reference voltage for boosting).
		volatile unsigned short		HVLD		: 1;	///< This bit sets the LCD voltage regulator into heavy load protection mode.
		volatile unsigned short		BISEL		: 1;	///< This bit selects the LCD drive bias.
		volatile unsigned short		BSTEN		: 1;	///< This bit turns the LCD voltage booster on and off.
		volatile unsigned short		reserved_1	: 3;	///< Reserved.
		volatile unsigned short		LC			: 5;	///< These bits set the LCD panel contrast.
		volatile unsigned short		reserved_2	: 3;	///< Reserved.
	} bit;
} LCD24PWR_REG;

/// LCD24A Display Control Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		DSPC		: 2;	///< These bits control the LCD display on/off and select a display mode.
		volatile unsigned short		DSPAR		: 1;	///< This bit switches the display area in the display data RAM.
		volatile unsigned short		reserved_1	: 1;	///< Reserved.
		volatile unsigned short		DSPREV		: 1;	///< This bit controls black/white inversion on the LCD display.
		volatile unsigned short		COMREV		: 1;	///< This bit selects the common pin assignment direction.
		volatile unsigned short		SEGREV		: 1;	///< This bit selects the segment pin assignment direction.
		volatile unsigned short		reserved_2	: 9;	///< Reserved.
	} bit;
} LCD24DSP_REG;

/// LCD24A COM Pin Control Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		PIN0		: 1;	///< These bits configure the partial drive of the COMx pins(0).
		volatile unsigned short		PIN1		: 1;	///< These bits configure the partial drive of the COMx pins(1).
		volatile unsigned short		PIN2		: 1;	///< These bits configure the partial drive of the COMx pins(2).
		volatile unsigned short		PIN3		: 1;	///< These bits configure the partial drive of the COMx pins(3).
		volatile unsigned short		PIN4		: 1;	///< These bits configure the partial drive of the COMx pins(4).
		volatile unsigned short		PIN5		: 1;	///< These bits configure the partial drive of the COMx pins(5).
		volatile unsigned short		PIN6		: 1;	///< These bits configure the partial drive of the COMx pins(6).
		volatile unsigned short		PIN7		: 1;	///< These bits configure the partial drive of the COMx pins(7).
		volatile unsigned short		PIN8		: 1;	///< These bits configure the partial drive of the COMx pins(8).
		volatile unsigned short		PIN9		: 1;	///< These bits configure the partial drive of the COMx pins(9).
		volatile unsigned short		PIN10		: 1;	///< These bits configure the partial drive of the COMx pins(10).
		volatile unsigned short		PIN11		: 1;	///< These bits configure the partial drive of the COMx pins(11).
		volatile unsigned short		PIN12		: 1;	///< These bits configure the partial drive of the COMx pins(12).
		volatile unsigned short		PIN13		: 1;	///< These bits configure the partial drive of the COMx pins(13).
		volatile unsigned short		PIN14		: 1;	///< These bits configure the partial drive of the COMx pins(14).
		volatile unsigned short		PIN15		: 1;	///< These bits configure the partial drive of the COMx pins(15).
	} bit;
} LCD24COMCX_REG;

/// LCD24A Interrupt Flag Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		FRMIF		: 1;	///< This bit indicates the frame interrupt cause occurrence status.
		volatile unsigned short		reserved_1	:15;	///< Reserved.
	} bit;
} LCD24INTF_REG;

/// LCD24A Interrupt Enable Register.
typedef union {
	volatile unsigned short			reg;
	struct {
		volatile unsigned short		FRMIE		: 1;	///< This bit enables the frame interrupt.
		volatile unsigned short		reserved_1	:15;	///< Reserved.
	} bit;
} LCD24INTE_REG;


#ifdef __cplusplus
}
#endif

#endif	// LCD24A_REG_H__

