/** *************************************************************************
 *
 * @file	main.c
 * @brief	This is PPG monitor program for S1C17W23 application note.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 07/20/2016    1.00
 *				- First revision.
 *			- 08/25/2016    1.10
 *				- 2nd revision.
 *
 ************************************************************************* */

#include "crt0.h"
#include "init.h"
#include "ad12.h"
#include "osc.h"
#include "t16_ch0.h"
#include "t16_ch1.h"
#include "t16_ch3.h"
#include "ppg_mon.h"
#include "uart.h"

/* --- interrupt handler (GNU17v3 Only) --- */
#ifdef CRT0_LINKED
C17_INTERRUPT_HANDLER(_vector10_handler, intUartCh0);	// UART Ch.0 interrupt
C17_INTERRUPT_HANDLER(_vector09_handler, intT16Ch0);	// T16 Ch.0 interrupt
C17_INTERRUPT_HANDLER(_vector26_handler, intAdc12Ch0);	// ADC12A interrupt
#endif /* CRT0_LINKED */


/*
 * P10	J1-31	O	0: LED OFF  1: LED ON
 * P15	J1-26	O	0: Hold  1: Sampling
 * P16	J1-25	O	0: High Gain  1: Low Gain
 * P17	J1-24	O	0: Vref OFF  1: Vref ON
 * P20	J1-22	O	0: LPF OFF  1: LPF ON
 *
 * P36	J1-8	USIN0
 * P37	J1-7	USOUT0
 *
 * P12	J1-29	ADIN00
 * P13	J1-28	VREFA0
 *
 * P13	J1-28	#BZOUT	... There must not be influence.
 * P14	J1-27	BZOUT
 *
 *
 * UART protocol
 *
 *   [0] Command		Measure start command = 0x02 (STX)
 *   [1] Intermittent	0: continuous  1: intermittent
 *   [2] highGain		0: low gain  1: high gain
 *   [3] LPF			0: LPF OFF  1: LPF ON
 *   [4] filter1		Filter1  0: None  1: FIR  2: CC
 *   [5] filter2		Filter2  0: None  1: FIR  2: CC
 *   [6] skipNumL		Skipping first number
 *   [7] skipNumH
 *   [8] DataSetNumL	Data set number
 *   [9] DataSetNumH
 *  [10] ScanIntervalL	Sampling interval coefficient.  If given = 2,499 --> ( 2,499 + 1 ) / ( 4MHz / 16 ) = 10msec.
 *  [11] ScanIntervalH
 *  [12] SamplingDelayL	Sampling Delay coefficient.  If given = 79 --> ( 79 + 1 ) / ( 4MHz / 1 ) = 20usec.
 *  [13] SamplingDelayH
 *  [14] Frame number for FIR
 *  [15-44] FIR parameters
 *
 *
 * Peripherals	(Interrupt level)
 *
 *   T16Ch0		(1)		Interval timer for sampling.
 *   T16Ch1		(-)		Timer for sampling delay and pulse width.
 *   T16Ch3		(-)		Clock for ADC12A.
 *   Adc12aCh0	(3)		Convert photo diode signal level.
 *   UartCh0	(5)		For serial communication.
 *
 */

#define OSC3_FREQ						OSC3_4MHZ
#define SAMPLING_DIV					4				// Clock division ratio selection (setting 1/16)
#define SAMPLING_DELEY_DIV				0				// Clock division ratio selection (setting 1/1)
#define ADC12_CLOCK_COEF				1				// ( 4MHz / 1 ) / ( 1 + 1 ) = 2MHz ... ADC12A: fADCLK(max) = 2,000kHz

#define MEAS_START_COMMAND				0x02			// STX
#define MEANS_NO_DATA					29999			// means "no data".


int main(void);
void execFir(void);
void calcCC(void);
void initUartCh0(void);
void initT16Ch0(void);
void initT16Ch1(void);
void initT16Ch3(void);

void intUartCh0(void)__attribute__ ((interrupt_handler));
void intT16Ch0(void)__attribute__ ((interrupt_handler));
void intAdc12Ch0(void)__attribute__ ((interrupt_handler));


PPG_MON pm;


int main (void)
{

	/// Set interrupt level.
	ILVT16_0		= 1;		// Set T16Ch0 interrupt level
	ILVADC12_0		= 3;		// Set Adc12aCh0 interrupt level;  must be > ILVT16_0
	ILVUART_0		= 5;		// Set UartCh0 interrupt level;  must be > ILVADC12_0

	changeSysClkOscto3( OSC3_FREQ );	// Change clock to OSC3
	stopIOsc ();						// Stop IOSC

	initT16Ch0();				// Initialize T16Ch0 for interval timer
	initT16Ch1();				// Initialize T16Ch1 for delay timer
	initT16Ch3();				// Initialize T16Ch3 for ADC12A's clock
	initUartCh0();				// Initialize UartCh0 for PC communication

	// Assign port to ADC12.
	P1MODSEL		= 0x000C;	// ----.----.----.11--	P13=VREFA0 P12=ADIN00
	P1FNCSEL		= 0x00A0;	// ----.----.1010.----

	initAdc();					// Initialize Adc12aCh0 for photo diode signal level converter

	ADC12_0TRG	= 0x0014;		// --00.0000.0001.-100	SMPCLK=4(8cycle) CNVTRG=1(T16) CNVMD=0(once) STMD=0(right) STAAIN=0 ENDAIN=0

	// Set P10, P15, P16, P17, P20 for output.
	P1IOEN			= 0x00E1;	// ----.----.111-.---1
	P2IOEN_P20OEN	= 1;

	while (1) {
		asm("ei");
		asm("halt");
		asm("nop");
	}

	return (0);
}

void execFir(void)
{
	signed short	ptr;
	unsigned short	i;
	signed long	res, sum = 0;

	ptr = pm.dataPtr;
	if ( pm.filter1 == 1 ) {	// Filter1 = FIR
		for ( i = 0; i < pm.firNum; i++ ) {
			if ( pm.firCoef[i] >= 0 ) {
				res = pm.rawData[ptr];
				res *= pm.firCoef[i];
	//			res = pm.rawData[ptr] * pm.firCoef[i];	// NG
				sum += res >> 16;
			} else {
				res = pm.rawData[ptr];
				res *= ( -pm.firCoef[i] );
	//			res = pm.rawData[ptr] * ( -pm.firCoef[i] );	// NG
				sum -= res >> 16;
			}
			if ( --ptr < 0 ) {
				ptr = DATA_NUM - 1;
			}
		}
	} else {					// Filter1 = CC, Filter2 = FIR
		for ( i = 0; i < pm.firNum; i++ ) {
			if ( pm.firCoef[i] >= 0 ) {
				res = pm.ccRes[ptr];
				res *= pm.firCoef[i];
	//			res = pm.rawData[ptr] * pm.firCoef[i];	// NG
				sum += res >> 16;
			} else {
				res = pm.ccRes[ptr];
				res *= ( -pm.firCoef[i] );
	//			res = pm.rawData[ptr] * ( -pm.firCoef[i] );	// NG
				sum -= res >> 16;
			}
			if ( --ptr < 0 ) {
				ptr = DATA_NUM - 1;
			}
		}
	}
	pm.firRes[pm.dataPtr] = sum;
}

void calcCC(void)
{
	unsigned short	i;
	signed short	t, y;

	if ( ++pm.availDataNum >= pm.setNum ) {
    	pm.availDataNum = pm.setNum;
		y = 0;
		t = pm.dataPtr - pm.setNum + 1;
		if ( t < 0 ) {
			t += DATA_NUM;
		}
    	if ( pm.filter1 == 1 ) {	// Filter1 = CC
			for ( i = 0; i < pm.numFrame; i++ ) {
				y -= pm.rawData[t];
				if ( ++t >= DATA_NUM ) {
					t = 0;
				}
			}
			for ( i = 0; i < ( 2 * pm.numFrame ); i++ ) {
				y += pm.rawData[t];
				if ( ++t >= DATA_NUM ) {
					t = 0;
				}
			}
			for ( i = 0; i < pm.numFrame; i++ ) {
				y -= pm.rawData[t];
				if ( ++t >= DATA_NUM ) {
					t = 0;
				}
			}
    	} else {					// Filter1 = FIR, Filter2 = CC
			for ( i = 0; i < pm.numFrame; i++ ) {
				y -= pm.firRes[t];
				if ( ++t >= DATA_NUM ) {
					t = 0;
				}
			}
			for ( i = 0; i < ( 2 * pm.numFrame ); i++ ) {
				y += pm.firRes[t];
				if ( ++t >= DATA_NUM ) {
					t = 0;
				}
			}
			for ( i = 0; i < pm.numFrame; i++ ) {
				y -= pm.firRes[t];
				if ( ++t >= DATA_NUM ) {
					t = 0;
				}
			}
    	}
		pm.ccRes[pm.dataPtr] = y;
	}
}

void initUartCh0 (void)
{
	/*
	 * CLK_UART = 4 MHz
	 *
	 *   bps = CLK_UART / ( ( BRT + 1) * 16 + FMD ) = 4,000,000 / ( ( 0 + 1 ) * 16 + 15 ) = 129,032  ( target:  128,000)
	 *   BRT = 0,  FMD = 15  err = ( 129,032 - 128,000 ) / 128,000 = 0.81 %
	 *
	 */

	// Set UPMUX for UART
	P3UPMUX3_P36PERISEL	= 3;			/// - Peripheral circuit = UART.
	P3UPMUX3_P36PERICH	= 0;			/// - Peripheral circuit channel number = 0.
	P3UPMUX3_P36PPFNC	= 1;			/// - Peripheral I/O function to be assigned to the port = USIN.
	P3UPMUX3_P37PERISEL	= 3;			/// - Peripheral circuit = UART.
	P3UPMUX3_P37PERICH	= 0;			/// - Peripheral circuit channel number = 0.
	P3UPMUX3_P37PPFNC	= 2;			/// - Pperipheral I/O function to be assigned to the port = USOUT.

	/// Stop UART.
	stopUart();

	/// Configure operating clock.
	UA0CLK_CLKSRC		= 2;			/// - Select the clock source of the UART(OSC3=4MHz).
	UA0CLK_CLKDIV		= 0;			/// - Select the division ratio(1/1).
	UA0CLK_DBRUN		= 0;			/// - Clock supplied in DEBUG mode.

	/// The clock source operations in SLEEP mode.
	CLGOSC_OSC3SLPC		= 0;			/// - Continue operation state before SLEEP.

	/// Configure the following UA0MOD register bits.
	UA0MOD_PUEN			= 1;			/// - Set USIN0 pin(0=Disable pull-up / 1=Enable pull-up).
	UA0MOD_OUTMD		= 0;			/// - Set USOUT0 pin(0=Push-pull output / 1=open-drain output).
	UA0MOD_IRMD			= 0;			/// - IrDA interface(0=Disable / 1=Enable).
	UA0MOD_CHLN			= 1;			/// - Data length(0=7-bit  / 1=8-bit).
	UA0MOD_PREN			= 0;			/// - Parity function(0=Disable / 1=Enable).
	UA0MOD_PRMD			= 0;			/// - Even parity selection(0=Unused / 1=Used).
	UA0MOD_STPB			= 0;			/// - Stop bit length(0=1-bit  / 1=2-bit).

	/// Set transfer rate( bps = CLK_UART / {(BRT + 1) x 16 + FMD} ).
	/// - S1C17W23 :  128,000 bps  =>  4,000,000 / ( ( 0 + 1 ) * 16 + 15 ) = 129,032
	UA0BR_BRT			= 0;			/// - BRT : 0
	UA0BR_FMD			= 15;			///	- FMD : 15

	/// Start UART
	startUart();

	/// Set for interrupt.
	UA0INTF				= 0x037f;		//// Clear all flag.
	UA0INTE				= 0x0002;		//// Enable RB1FIE

	// Set port for peripheral.
	P3FNCSEL_P36MUX		= 1;			/// - Select the peripheral I/O function to be assigned to P36.
	P3MODSEL_P36SEL		= 1;			/// - Select P36 is used for a peripheral I/O function.
	P3FNCSEL_P37MUX		= 1;			/// - Select the peripheral I/O function to be assigned to P37.
	P3MODSEL_P37SEL		= 1;			/// - Select P37 is used for a peripheral I/O function.
}

void initT16Ch0 (void)
{
	/// Stop T16 Ch0 Timer.
 	stopT16Ch0();

	T16_0CTL_MODEN		= 0;					/// - Stop supplying operating clock

	/// Configure the T16 Ch0 operating clock.
	T16_0CLK_CLKSRC 	= 2;					/// - Clock source selection(setting OSC3)
	T16_0CLK_CLKDIV 	= SAMPLING_DIV;			/// - Clock division ratio selection
	T16_0CLK_DBRUN		= 0;					/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_0CTL_MODEN		= 1;					/// - Start supplying operating clock.

	/// Select operation mode.
	T16_0MOD_TRMD		= 0;					/// - Set repeat mode.

	/// initialize interrupt.
	T16_0INTF			= 0x0001;				/// - Clear interrupt flag(T16_0INTF_UFIF = 1).
	T16_0INTE_UFIE		= 0;					/// - Disable interrupt.

	return;
}

void initT16Ch1 (void)
{
	/// Stop T16 Ch1 Timer.
 	stopT16Ch1();

	T16_1CTL_MODEN		= 0;					/// - Stop supplying operating clock

	/// Configure the T16 Ch1 operating clock.
	T16_1CLK_CLKSRC 	= 2;					/// - Clock source selection(setting OSC3)
	T16_1CLK_CLKDIV 	= SAMPLING_DELEY_DIV;	/// - Clock division ratio selection
	T16_1CLK_DBRUN		= 0;					/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_1CTL_MODEN		= 1;					/// - Start supplying operating clock.

	/// Select operation mode.
	T16_1MOD_TRMD		= 1;					/// - Set one shot mode.

	/// initialize interrupt.
	T16_1INTF			= 0x0001;				/// - Clear interrupt flag(T16_1INTF_UFIF = 1).
	T16_1INTE_UFIE		= 0;					/// - Disable interrupt.

	return;
}

void initT16Ch3 (void)
{
	/// Stop T16 Ch3 Timer.
 	stopT16Ch3();

	T16_3CTL_MODEN		= 0;					/// - Stop supplying operating clock

	/// Configure the T16 Ch3 operating clock.
	T16_3CLK_CLKSRC 	= 2;					/// - Clock source selection(setting OSC3)
	T16_3CLK_CLKDIV 	= 0;					/// - Clock division ratio selection
	T16_3TR				= ADC12_CLOCK_COEF;		/// - Reload data
	T16_3CLK_DBRUN		= 0;					/// - No clock supplied in DEBUG mode

	/// Enable count operation clock.
	T16_3CTL_MODEN		= 1;					/// - Start supplying operating clock.

	/// Select operation mode.
	T16_3MOD_TRMD		= 0;					/// - Set repeat mode.

	/// initialize interrupt.
	T16_3INTF			= 0x0001;				/// - Clear interrupt flag(T16_3INTF_UFIF = 1).
	T16_3INTE_UFIE		= 0;					/// - Disable interrupt.

	return;
}

void intUartCh0 (void)
{
	/*
	 *   [0] Command		Measure start command = 0x02 (STX)
	 *   [1] Intermittent	0: continuous  1: intermittent
	 *   [2] highGain		0: low gain  1: high gain
	 *   [3] LPF			0: LPF OFF  1: LPF ON
	 *   [4] filter1		Filter1  0: None  1: FIR  2: CC
	 *   [5] filter2		Filter2  0: None  1: FIR  2: CC
	 *   [6] skipNumL		Skipping first number
	 *   [7] skipNumH
	 *   [8] DataSetNumL	Data set number
	 *   [9] DataSetNumH
	 *  [10] ScanIntervalL	Sampling interval coefficient.  If given = 2,499 --> ( 2,499 + 1 ) / ( 4MHz / 16 ) = 10msec.
	 *  [11] ScanIntervalH
	 *  [12] SamplingDelayL	Sampling Delay coefficient.  If given = 79 --> ( 79 + 1 ) / ( 4MHz / 1 ) = 20usec.
	 *  [13] SamplingDelayH
	 *  [14] Frame number for FIR
	 *  [15-44] FIR parameters
	 *
	 */

	unsigned short	i, i1;

	if ( ( UA0INTF & 0x0002 ) != 0 ) {		// receive buffer 1 byte full interrupt

		UA0INTF = 0x0002;					// clear interrupt flag

		if ( pm.step == STEP_WAITING_COMMAND ) {

			receiveUartData( pm.rcvData, CONTROL_HEADER_LEN );

			if ( pm.rcvData[0] == MEAS_START_COMMAND ) {

				unsigned short	filter1, filter2;

				pm.interm = pm.rcvData[1];
				pm.highGain = pm.rcvData[2];
				pm.lpf = pm.rcvData[3];
				filter1 = pm.rcvData[4];
				filter2 = pm.rcvData[5];
				pm.skipNum = pm.rcvData[6] + ( ( (unsigned short) pm.rcvData[7] ) << 7 );
				pm.dataSetNum = pm.rcvData[8] + ( ( (unsigned short) pm.rcvData[9] ) << 7 );
				pm.scanInterval = pm.rcvData[10] + ( ( (unsigned short) pm.rcvData[11] ) << 7 );
				pm.samplingDelay = pm.rcvData[12] + ( ( (unsigned short) pm.rcvData[13] ) << 7 );
				pm.numFrame = pm.rcvData[14];

				pm.setNum = 4 * pm.numFrame;

				pm.firNum = pm.rcvData[15];
				i1 = 16;
				for ( i = 0; i < pm.firNum; i++ ) {
					pm.firCoef[i] = pm.rcvData[i1] + ( ( (unsigned short) pm.rcvData[i1 + 1] + ( ( (unsigned short) pm.rcvData[i1 + 2]) << 7 ) ) << 7 );
					i1 += 3;
				}

				presetT16Ch0Count(pm.scanInterval);

				switch ( filter1 ) {
				case 0:			// None
					if ( filter2 == 1 ) {				// None + FIR  : FIR
						pm.filter1 = 1;		// FIR
						pm.filter2 = 0;		// None
					} else if ( filter2 == 2 ) {		// None + CC   : CC
						pm.filter1 = 2;		// CC
						pm.filter2 = 0;		// None
					} else {							// None + None : None
						pm.filter1 = 0;		// None
						pm.filter2 = 0;		// None
					}
					break;

				case 1:			// FIR
					if ( filter2 == 1 ) {				// FIR  + FIR  : FIR
						pm.filter1 = 1;		// FIR
						pm.filter2 = 0;		// None
					} else if ( filter2 == 2 ) {		// FIR  + CC   : FIR -> CC
						pm.filter1 = 1;		// FIR
						pm.filter2 = 2;		// CC
					} else {							// FIR  + None : FIR
						pm.filter1 = 1;		// FIR
						pm.filter2 = 0;		// None
					}
					break;

				case 2:			// CC: calculate cross-correlation value
					if ( filter2 == 1 ) {				// CC   + FIR  : CC -> FIR
						pm.filter1 = 2;		// CC
						pm.filter2 = 1;		// FIR
					} else if ( filter2 == 2 ) {		// CC   + CC   : CC
						pm.filter1 = 2;		// CC
						pm.filter2 = 0;		// None
					} else {							// CC   + None : CC
						pm.filter1 = 2;		// CC
						pm.filter2 = 0;		// None
					}
					break;

				}

				if ( pm.interm ) {		// 1: intermittent
					pm.p1xBase = 0x8000;	// 1-0-.---0.----.----  P10=0(LED OFF)  P15=0(Hold)  P17=1(Vref ON)
				} else {				// 0: continuous
					pm.p1xBase = 0xA100;	// 1-1-.---1.----.----  P10=1(LED ON)  P15=1(Sampling)  P17=1(Vref ON)
				}
				if ( pm.highGain ) {	// 1: high gain
					pm.p1xBase &= ~0x4000;	// -0--.----.----.----  P16=0(High Gain)
				} else {				// 0: low gain
					pm.p1xBase |= 0x4000;	// -1--.----.----.----  P16=1(Low Gain)
				}
				P1DAT	= pm.p1xBase;

				if ( pm.lpf ) {			// 1: noise filter ON
					P2DAT_P20OUT	= 0x01;				// P20	1: LPF ON
				} else {				// 0: noise filter OFF
					P2DAT_P20OUT	= 0x00;				// P20	0: LPF OFF
				}

				if ( pm.skipNum == 0 ) {		// No skipped first
					pm.step = STEP_MEASURING;
				} else {
					pm.step = STEP_SKIPPING_FIRST;
				}
				pm.stage = 0;
				pm.measCount = 0;
				pm.skipCount = 0;
				pm.dataPtr = 0;
				pm.availDataNum = 0;
				startT16Ch0();
			}

		} else {

			unsigned short	dummy;

			dummy	= UA0RXD_RXD;	// recite

		}
	}
}

void intT16Ch0 (void)
{
	unsigned short	ccVal;

	T16_0INTF = 0x0001;							// clear interrupt flag

	if ( pm.step == STEP_SKIPPING_FIRST ) {
		if ( ++pm.skipCount >= pm.skipNum ) {
			pm.step = STEP_MEASURING;
		}
	}

	startT16Ch3();

	/*
	 * P10	J1-31	0: LED OFF  1: LED ON
	 * P15	J1-26	0: Hold  1: Sampling
	 * P16	J1-25	0: High Gain  1: Low Gain
	 * P17	J1-24	0: Vref OFF  1: Vref ON
	 * P20	J1-22	0: LPF OFF  1: LPF ON
	 */

	if ( pm.interm ) {		// 1: intermittent

		P1DAT		= ( pm.p1xBase | 0x2100 );	// --1-.---1.----.----  P10=1(LED ON)  P15=1(Sampling)

		// delay for sample dark level
		if ( pm.samplingDelay != 0 ) {
			presetT16Ch1Count(pm.samplingDelay);
			startT16Ch1();
			while ( T16_1CTL_PRUN != 0 ) {
			}
		}

		// measure
		ADC12_0INTF = 0x0001;				// ----.----.----.---1	AD0CIF=1(clear)
		startAdc();

		do {
			asm("ei");
			asm("halt");
			asm("nop");
		} while ( pm.stage != 1 );

		stopAdc();
		P1DAT		= ( pm.p1xBase & ~0x2100 );	// --0-.---0.----.----  P10=0(LED OFF)  P15=0(Hold)

	} else {		// 0: continuous

		// measure
		ADC12_0INTF = 0x0002;				// ----.----.----.--10	AD0CIF=2(clear)
		startAdc();

		do {
			asm("ei");
			asm("halt");
			asm("nop");
		} while ( pm.stage != 1 );

		stopAdc();

	}

	pm.stage = 0;
	stopT16Ch3();

	pm.rawData[pm.dataPtr] = pm.measuredVal;

	pm.sndData[0] = pm.measuredVal & 0xff;			// Lower 8bit
	pm.sndData[1] = pm.measuredVal >> 8;			// Higher 8bit

	if ( pm.filter1 == 1 ) {			// FIR
		execFir();
	} else if ( pm.filter1 == 2 ) {		// CC
		calcCC();
	}
	if ( pm.filter2 == 1 ) {			// FIR
		execFir();
	} else if ( pm.filter2 == 2 ) {		// CC
		calcCC();
	}

	if ( pm.step == STEP_MEASURING ) {
		if ( pm.filter1 == 1 || pm.filter2 == 1 ) {		// FIR
			pm.sndData[2] = pm.firRes[pm.dataPtr] & 0xff;		// Lower 8bit
			pm.sndData[3] = pm.firRes[pm.dataPtr] >> 8;			// Higher 8bit
		} else {
			pm.sndData[2] = MEANS_NO_DATA & 0xff;				// Lower 8bit
			pm.sndData[3] = MEANS_NO_DATA >> 8;					// Higher 8bit
		}
		if ( pm.filter1 == 2 || pm.filter2 == 2 ) {		// CC
			ccVal = pm.ccRes[pm.dataPtr] + 0x8000;
			pm.sndData[4] = ccVal & 0xff;						// Lower 8bit
			pm.sndData[5] = ( ccVal >> 8 ) & 0xff;
		} else {
			pm.sndData[4] = MEANS_NO_DATA & 0xff;				// Lower 8bit
			pm.sndData[5] = MEANS_NO_DATA >> 8;					// Higher 8bit
		}
		sendUartData( pm.sndData, 6 );
	}

	if ( ++pm.dataPtr >= DATA_NUM ) {
		pm.dataPtr = 0;
	}

	if ( pm.step == STEP_MEASURING ) {
		pm.measCount++;
		if ( pm.measCount >= pm.dataSetNum ) {		// completed
			stopT16Ch0();
			P1DAT		= 0x4000;	// 010-.---0.----.----  P10=0(LED OFF)  P15=0(Hold)  P16=1(Low Gain)  P17=0(Vref OFF)
			pm.step = STEP_WAITING_COMMAND;
		}
	}

}

void intAdc12Ch0 (void)
{
	if ( ( ADC12_0INTF & 0x0100 ) != 0 ) {	// ----.---1.----.----	AD0OVIE==1

		// Trap overwrite error, then clear interrupt flags and restart ADC.
		ADC12_0INTF = 0x0101;				// ----.---1.----.---1	AD0OVIE=1(clear), AD0CIF=1(clear)
		startAdc();							///Start ADC12

	} else if ( ( ADC12_0INTF & 0x0001 ) != 0 ) {	// ----.----.----.---1	AD0CIF==1

		// Complete ADC.
		ADC12_0INTF = 0x0001;				// ----.----.----.---1	AD0CIF=1(clear)

		// read ADC.
		pm.measuredVal = ADC12_0AD0D;

		pm.stage = 1;

	}

	return;

}

