/** *************************************************************************
 *
 * @file	uart.c
 * @brief	This is UART driver file for S1C17W22_W23.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 04/25/2014    1.00
 *				- First revision.
 *
 ************************************************************************* */

#include "uart.h"


#if 0

/** *************************************************************************
 * initUart function.
 *
 * @brief	initialize UART.
 *
 * @retval	STATUS_OK	Success.
 * @retval	STATUS_NG	Failed.
 ************************************************************************* */
int initUart( void )
{
	/// When IOSC doesn't start, nothing is done.
	if ( CLGOSC_IOSCEN == 0 ) {
		return STATUS_NG;
	}

	/// Stop UART.
	stopUart();

	/// Initialize the UART input/output function to the ports.
	P0IOEN_P01IEN		= 0;			/// - P01 : Disable input.
	P0IOEN_P01OEN		= 0;			/// - P01 : Disable output.
	P0MODSEL_P01SEL		= 0;			/// - P01 : Disable peripheral I/O function.
	P0IOEN_P02IEN		= 0;			/// - P02 : Disable input.
	P0IOEN_P02OEN		= 0;			/// - P02 : Disable output.
	P0MODSEL_P02SEL		= 0;			/// - P02 : Disable peripheral I/O function.

#ifdef PPORT_M01
	/// Assign the UART input/output function to the ports.
	P0FNCSEL_P01MUX		= 0;			/// - P01 : UART(USIN0).
	P0MODSEL_P01SEL		= 1;			/// - P01 : Enable peripheral I/O function.
	P0FNCSEL_P02MUX		= 0;			/// - P02 : UART(USOUT0).
	P0MODSEL_P02SEL		= 1;			/// - P02 : Enable peripheral I/O function.
#endif // PPORT_M01

#ifdef PPORT_W23
	/// Assign the UART input/output function to the ports.
	P0FNCSEL_P01MUX		= 1;			/// - P01 : UPMUX.
	P0MODSEL_P01SEL		= 1;			/// - P01 : Enable peripheral I/O function.
	P0FNCSEL_P02MUX		= 1;			/// - P02 : UPMUX.
	P0MODSEL_P02SEL		= 1;			/// - P02 : Enable peripheral I/O function.

	/// Set Universal Port Multiplexer (UPMUX).
	P0UPMUX0_P01PERISEL	= 3;			/// - P01 : UART.
	P0UPMUX0_P01PPFNC	= 1;			/// - P01 : USIN.
	P0UPMUX0_P01PERICH	= 0;			/// - P01 : Ch.0.
	P0UPMUX1_P02PERISEL	= 3;			/// - P02 : UART.
	P0UPMUX1_P02PPFNC	= 2;			/// - P02 : USOUT.
	P0UPMUX1_P02PERICH	= 0;			/// - P02 : Ch.0.
#endif // PPORT_W23

	/// Configure operating clock.
	UA0CLK_CLKSRC		= 0;			/// - Select the clock source of the UART(IOSC=7.37MHz).
	UA0CLK_CLKDIV		= 0;			/// - Select the division ratio(1/1).
	UA0CLK_DBRUN		= 1;			/// - Clock supplied in DEBUG mode.

	/// The clock source operations in SLEEP mode.
	CLGOSC_IOSCSLPC		= 0;			/// - Continue operation state before SLEEP.

	/// Configure the following UA0MOD register bits.
	UA0MOD_PUEN			= 1;			/// - Set USIN0 pin(0=Disable pull-up / 1=Enable pull-up).
	UA0MOD_OUTMD		= 0;			/// - Set USOUT0 pin(0=Push-pull output / 1=open-drain output).
	UA0MOD_IRMD			= 0;			/// - IrDA interface(0=Disable / 1=Enable).
	UA0MOD_CHLN			= 1;			/// - Data length(0=7-bit  / 1=8-bit).
	UA0MOD_PREN			= 0;			/// - Parity function(0=Disable / 1=Enable).
	UA0MOD_PRMD			= 0;			/// - Even parity selection(0=Unused / 1=Used).
	UA0MOD_STPB			= 0;			/// - Stop bit length(0=1-bit  / 1=2-bit).

	/// Set transfer rate( bps = CLK_UART / {(BRT + 1) x 16 + FMD} ).
	/// - S1C17M01     : 460800 bps = 7.37MHz / 16 = 460625(-0.038%).
	/// - S1C17W22_W23 :  43750 bps =  700KHz / 16 = 43750.
	UA0BR_BRT			= 0x0;			/// - BRT : 0
	UA0BR_FMD			= 0x0;			///	- FMD : 0

	/// Set interrupt level.
	ILVUART_0			= INTERRUPT_LEVEL0;	/// - Set interrupt level-0.

	return STATUS_OK;
}

#endif


/** *************************************************************************
 * startUart function.
 *
 * @brief	Set UART enable.
 ************************************************************************* */
void startUart( void )
{
	/// Execute software reset.
	UA0CTL_SFTRST		= 1;
	while( UA0CTL_SFTRST == 1 ) {
		// Wait...
	}

	/// Enable UART Ch.0 operations.
	UA0CTL_MODEN		= 1;
}


/** *************************************************************************
 * stopUart function.
 *
 * @brief	Set UART enable.
 ************************************************************************* */
void stopUart( void )
{
	/// Disable UART Ch.0 operations.
	UA0CTL_MODEN		= 0;

	/// Execute software reset.
	UA0CTL_SFTRST		= 1;
	while( UA0CTL_SFTRST == 1 ) {
		// Wait...
	}
}


/** *************************************************************************
 * sendUartData function.
 *
 * @brief	UART transmit data write.
 *
 * @param	data	Pointer of transmit data.
 * @param	size	Transmit data size.
 *
 * @retval	STATUS_OK	Success.
 * @retval	STATUS_NG	Failed.
 ************************************************************************* */
int sendUartData( unsigned char *data, unsigned short size )
{
	unsigned short i;

	// Send data.
	for ( i = 0 ; i < size ; i++ ) {

		/// Check Transmit buffer empty interrupt.
		while ( UA0INTF_TBEIF == 0 ) {
			// Wait...
		}

		/// Send data.
		UA0TXD_TXD = data[i];
	}

	return STATUS_OK;
}


/** *************************************************************************
 * receiveUartData function.
 *
 * @brief	UART receive data read.
 *
 * @param	data	Pointer of receive data.
 * @param	size	Receive data size.
 *
 * @retval	STATUS_OK	Success.
 * @retval	STATUS_NG	Failed.
 ************************************************************************* */
int receiveUartData( unsigned char *data, unsigned short size )
{
	unsigned short i;

	/// Receive data.
	for ( i = 0 ; i < size ; i++ ) {

		/// Check receive buffer one byte full interrupt.
		while ( UA0INTF_RB1FIF == 0 ) {
			// Wait...
		}

		/// Read receive data.
		data[i]	= UA0RXD_RXD;
	}

	return STATUS_OK;
}
